from distutils.version import LooseVersion
from infinipy import System
from infinipy.system.exceptions import APICommandFailed
from logsetup import UserLogger

MAX_SYSTEM_SNAPSHOTS = 10000
MAX_VOL_SNAPSHOTS = 210
MAX_VOL_NAME_LEN = 46

MIN_SUPPORTED_VER = LooseVersion("1.5.0")
MAX_SUPPORTED_VER = LooseVersion("2.2.0")

log = UserLogger(__name__)

def connect_to_infinibox(config):
    hostname = config['infinibox']['host']
    username = config['infinibox']['username']
    password = config['infinibox']['password']

    log.uinfo("Connecting to the Infinibox '{}'".format(hostname))
    try:
        infinibox = System(hostname, api_port=80, username=username, password=password)
    except APICommandFailed, e:
        if e.code in ('AUTHENTICATION_REQUIRED', 'UNAUTHORIZED'):
            log.uabort("Couldn't connect to '{}' API: username/password error".format(hostname))
        else:
            log.uabort("Caught APICommandFailed")
    except Exception, e:
        log.uabort_with_exc_info("Failed to connect, aborting.")
    return infinibox


def infinibox_validate(infinibox):
    fail_msg = "Failed to connect to an API, aborting."
    try:
        infinibox_version = LooseVersion(infinibox.get_version())
        snapshots_count = sum(p._api.get('snapshots_count') for p in infinibox.get_pools())
    except APICommandFailed, e:
        if e.code in ('AUTHENTICATION_REQUIRED', 'UNAUTHORIZED'):
            log.uabort("Couldn't connect to infinibox API: username/password error.")
        else:
            log.uabort_with_exc_info(fail_msg)
    except Exception:
        log.uabort_with_exc_info(fail_msg)

    # Validate constraints
    if not MIN_SUPPORTED_VER < infinibox_version < MAX_SUPPORTED_VER:
        log.uabort("Infinibox version {} is not supported by this version of snaprotator".format(infinibox_version))

    if snapshots_count >= MAX_SYSTEM_SNAPSHOTS:
        log.uabort("Maximum number of snapshots per system had been reached ({}), rotation will not be performed", MAX_SYSTEM_SNAPSHOTS)


def volume_validate(infinibox, volume_name):
    log.uinfo("Checking for the '{}' volume".format(volume_name))

    volumes = infinibox.objects.Volume.find(name=volume_name)
    if not volumes:
        log.uabort("Failed to find volume '{}', aborting.".format(volume_name))

    volume = volumes[0]
    snapshots = volume.get_snapshots()

    if len(snapshots) >= MAX_VOL_SNAPSHOTS:
        log.uabort("Too many snapshots ({}) for volume '{}', aborting.".format(len(snapshots), volume_name))

    if len(volume.get_name()) > MAX_VOL_NAME_LEN:
        log.uabort("Volume '{}' name is longer than ({}) characters, aborting.".format(volume_name, MAX_VOL_NAME_LEN))

    log.debug("Volume validation - OK")
    return volume, snapshots
